local BG = LibStub("AceAddon-3.0"):GetAddon("Battleground")
local module = BG:NewModule("AlteracValley", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("Battleground")

local landmarks = BG.landmarks
local namedButtons = BG.namedButtons
local pName = UnitName("player")

module.captureTimer = 245

module.buttons = {
	right = {
		L["Iceblood Tower"],
		L["Iceblood Graveyard"],
		L["Tower Point"],
		L["Frostwolf Graveyard"],
		L["East Frostwolf Tower"],
		L["West Frostwolf Tower"],
		L["Frostwolf Relief Hut"],
		L["Coldtooth Mine"],
	},
	left = {
		L["Stonehearth Bunker"],
		L["Stonehearth Graveyard"],
		L["Icewing Bunker"],
		L["Stormpike Graveyard"],
		L["Dun Baldar North Bunker"],
		L["Dun Baldar South Bunker"],
		L["Stormpike Aid Station"],
		L["Irondeep Mine"],
	},
	top = {
		L["Battleground Start"],
	},
	bottom = {
		L["Snowfall Graveyard"],
	}
}
local chatFilters = {
	[L["Captain Balinda Stonehearth"]] = {
		[L["Begone, uncouth scum!  The Alliance shall prevail in Alterac Valley!"]] = L["%s has engaged %s."],
		[L["Filthy Frostwolf cowards! If you want a fight, you'll have to come to me!"]] = L["%s has reset %s."],
	},
	[L["Captain Galvangar"]] = {
		[L["Die!  Your kind has no place in Alterac Valley!"]] = L["%s has engaged %s."],
		[L["I'll never fall for that, fool! If you want a battle it will be on my terms and in my lair."]] = L["%s has reset %s."],
	},
	[L["Vanndar Stormpike"]] = {
		[L["Soldiers of Stormpike, your General is under attack! I require aid! Come! Come! Slay these mangy Frostwolf dogs."]] = L["%s has engaged %s."],
		[L["You'll never get me out of me bunker, heathens!"]] = L["%s has reset %s."],
	},
	[L["Drek'Thar"]] = {
		[L["Stormpike filth! In my keep?! Slay them all!"]] = L["%s has engaged %s."],
		[L["You seek to draw the General of the Frostwolf legion out from his fortress? PREPOSTEROUS!"]] = L["%s has reset %s."],
	},
}
local faction = {
	[L["Captain Balinda Stonehearth"]] = FACTION_HORDE,
	[L["Captain Galvangar"]] = FACTION_ALLIANCE,
	[L["Vanndar Stormpike"]] = FACTION_HORDE,
	[L["Drek'Thar"]] = FACTION_ALLIANCE
}

-- Reinforcments
local rAlliance = 0
local rHorde = 0

local function filterFunc(chatFrame, event, msg, author, ...)
	if author == L["Herald"] then
		if msg:match(FACTION_ALLIANCE) then
			module:SendMessage("Battleground_Message", msg, FACTION_ALLIANCE, chatFrame)
		elseif msg:match(FACTION_HORDE) then
			module:SendMessage("Battleground_Message", msg, FACTION_HORDE, chatFrame)
		elseif msg == L["The Stormpike General is dead!"] then
			module:SendMessage("Battleground_Message", msg, FACTION_HORDE, chatFrame)
		elseif msg == L["The Frostwolf General is dead!"] then
			module:SendMessage("Battleground_Message", msg, FACTION_ALLIANCE, chatFrame)
		else return end
		return true -- filter
	end
	if chatFilters[author] and chatFilters[author][msg] then
		module:SendMessage("Battleground_Message", chatFilters[author][msg]:format(faction[author] == FACTION_HORDE and L["The Horde"] or L["The Alliance"], author), faction[author], chatFrame)
		return true
	end
end

function module:OnEnable()
	rAlliance = 0
	rHorde = 0
	ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_YELL", filterFunc)
	self:RegisterMessage("Battleground_POIUpdate")
	self:RegisterEvent("UPDATE_WORLD_STATES")
	self:RegisterEvent("CHAT_MSG_ADDON")
	SendAddonMessage("cap", "r", "BATTLEGROUND") -- Capping request for timers
end

function module:OnDisable()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_MONSTER_YELL", filterFunc)
end

function module:CHAT_MSG_ADDON(event, prefix, message, chan, sender)
	if prefix ~= "cap" or sender == pName then return end
	if message == "r" then
		for k, loc in pairs(self.buttons) do
			for i, name in pairs(loc) do
				if namedButtons[name].expire and namedButtons[name].status and ( namedButtons[name].status == "horde_assaulted" or namedButtons[name].status == "alliance_assaulted" ) then
					SendAddonMessage("cap", format("%s@%d@%d@%s", name, 245, 245 - (namedButtons[name].expire - time()), namedButtons[name].status == "horde_assaulted" and "horde" or "alliance"), "WHISPER", sender)
				end
			end
		end
	else
		local name, duration, elapsed, color = strmatch(message, "^(.+)@(%d+)@(%d+)@(%a+)$")
		if name then
			local poi = namedButtons[name]
			if poi and not poi.expire then
				poi.expire = time() + (duration - elapsed)
				local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
				if poi.status == "alliance_assaulted" then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"] end
				poi.time:SetTextColor(color.r, color.g, color.b)
				poi:SetScript("OnUpdate", BG.OnUpdateCountDown)
			end
		end
	end
end

function module:Battleground_POIUpdate(message, name, desc, ti, button)
	if not button then return end
	if landmarks[ti][2] == "neutral" and landmarks[ti][1] == "tower" then button:SetAlpha(.6) end
end

function module:UPDATE_WORLD_STATES()
	local alliance = select(3, GetWorldStateUIInfo(1))
	local horde = select(3, GetWorldStateUIInfo(2))
	if not alliance or not horde then return end
	
	local nr = tonumber(alliance:match("([0-9]+)")) or 0
	if (rAlliance - nr) == 100 then self:SendMessage("Battleground_Message", L["%s has defeated %s."]:format(L["The Horde"], L["Captain Balinda Stonehearth"]), FACTION_HORDE, DEFAULT_CHAT_FRAME) end
	rAlliance = nr
	nr = tonumber(horde:match("([0-9]+)")) or 0
	if (rHorde - nr) == 100 then self:SendMessage("Battleground_Message", L["%s has defeated %s."]:format(L["The Alliance"], L["Captain Galvangar"]), FACTION_ALLIANCE, DEFAULT_CHAT_FRAME) end
	rHorde = nr
end
